@@ -19,6 +19,7 @@ examples = [ cursorMovementExample
            , saveRestoreCursorExample
            , clearExample
            , scrollExample
+           , screenBuffersExample
            , sgrColorExample
            , sgrOtherExample
            , cursorVisibilityExample
@@ -26,6 +27,7 @@ examples = [ cursorMovementExample
            , titleExample
            , getCursorPositionExample
            , getTerminalSizeExample
+           , getLayerColorExample
            ]
 
 main :: IO ()
@@ -221,6 +223,17 @@ scrollExample = do
   -- Line Two
   -- Line Three
 
+screenBuffersExample :: IO ()
+screenBuffersExample = do
+  replicateM_ 5 $ putStrLn "This message is on the Normal Screen Bufffer"
+  replicateM_ 5 pause
+  useAlternateScreenBuffer
+  replicateM_ 5 $ putStrLn "This message is on the Alternate Screen Bufffer"
+  replicateM_ 5 pause
+  useNormalScreenBuffer
+  replicateM_ 5 $ putStrLn "This message is continuing where we left off"
+  replicateM_ 5 pause
+
 sgrColorExample :: IO ()
 sgrColorExample = do
   let colors = enumFromTo minBound maxBound :: [Color]
@@ -427,3 +440,16 @@ getTerminalSizeExample = do
     Nothing -> putStrLn "Error: unable to get the terminal size\n"
   pause
   -- The size of the terminal is 25 rows by 80 columns.
+
+getLayerColorExample :: IO ()
+getLayerColorExample = do
+  fgResult <- getLayerColor Foreground
+  case fgResult of
+    Just fgCol -> putStrLn $ "The reported foreground color is:\n" ++
+      show fgCol ++ "\n"
+    Nothing -> putStrLn "Error: unable to get the foreground color\n"
+  bgResult <- getLayerColor Background
+  case bgResult of
+    Just bgCol -> putStrLn $ "The reported background color is:\n" ++
+      show bgCol ++ "\n"
+    Nothing -> putStrLn "Error: unable to get the background color\n"
