@@ -1,9 +1,10 @@
 #include "Common-Safe-Haskell.hs"
 
-{-| Through this module, this library provides platform-independent support for
+{-| == Introduction
+Through this module, this library provides platform-independent support for
 control character sequences following the \'ANSI\' standards (see further below)
 for terminal software that supports those sequences, running on a Unix-like
-operating system or Windows.
+operating system or on Windows (see further below).
 
 The sequences of control characters (also referred to as \'escape\' sequences or
 codes) provide a rich range of functionality for terminal control, which
@@ -21,6 +22,8 @@ includes:
 
  * Scrolling the screen up or down
 
+ * Switching between the Alternate and Normal Screen Buffers
+
  * Clickable hyperlinks to URIs
 
  * Changing the title of the terminal
@@ -29,23 +32,7 @@ A terminal that supports control character sequences acts on them when they
 are flushed from the output buffer (with a newline character @\"\\n\"@ or, for
 the standard output channel, @hFlush stdout@).
 
-The functions moving the cursor to an absolute position are 0-based (the
-top-left corner is considered to be at row 0 column 0) (see 'setCursorPosition')
-and so is 'getCursorPosition'. The \'ANSI\' standards themselves are 1-based
-(that is, the top-left corner is considered to be at row 1 column 1) and some
-functions reporting the position of the cursor are too (see
-'reportCursorPosition').
-
-The native terminal software on Windows is \'Command Prompt\' or \`PowerShell\`.
-Before Windows 10 version 1511 (known as the \'November [2015] Update\' or
-\'Threshold 2\') that software did not support such control sequences. For that
-software, this library also provides support for such sequences by using
-emulation.
-
-Terminal software other than the native software exists for Windows. One example
-is the \'mintty\' terminal emulator for \'Cygwin\', \'MSYS\' or \'MSYS2\', and
-dervied projects, and for \'WSL\' (Windows Subsystem for Linux).
-
+== \'ANSI\' standards
 The \'ANSI\' standards refer to (1) standard ECMA-48 \`Control Functions for
 Coded Character Sets\' (5th edition, 1991); (2) extensions in ITU-T
 Recommendation (previously CCITT Recommendation) T.416 (03/93) \'Information
@@ -59,35 +46,78 @@ current versions of Windows at
 
 The whole of the \'ANSI\' standards are not supported by this library but most
 (if not all) of the parts that are popular and well-supported by terminal
-software are supported. Every function exported by this module comes in three
-variants, namely:
+software are supported (see further below).
+
+== Cursor positions
+The functions moving the cursor to an absolute position are 0-based (the
+top-left corner is considered to be at row 0 column 0) (see 'setCursorPosition')
+and so is 'getCursorPosition'. The \'ANSI\' standards themselves are 1-based
+(that is, the top-left corner is considered to be at row 1 column 1) and some
+functions reporting the position of the cursor are too (see
+'reportCursorPosition').
+
+== Windows and control character sequences
+The native terminal software on Windows has developed over time. Before
+Windows 10 version 1511 (known as the \'November [2015] Update\' or
+\'Threshold 2\') that software did not support control character sequences. For
+that software, this library also provides support for such sequences by using
+emulation based on the Windows Console API. From 2018, Microsoft introduced the
+Windows Pseudo Console (\'ConPTY\') API and then Windows Terminal, with the
+objective of replacing most of the Windows Console API with the use of control
+character sequences and retiring the historical user-interface role of Windows
+Console Host (\'ConHost\').
+
+Terminal software other than the native software exists for Windows. One example
+is the \'mintty\' terminal emulator for \'Cygwin\', \'MSYS\' or \'MSYS2\', and
+dervied projects, and for \'WSL\' (Windows Subsystem for Linux).
+
+GHC's management of input and output (IO) on Windows has also developed over
+time. If they are supported by the terminal software, some control character
+sequences cause data to be emitted into the console input stream. For GHC's
+historical and default IO manager, the function 'hGetBufNonBlocking' in module
+"System.IO" does not work on Windows. This has been attributed to the lack of
+non-blocking primatives in the operating system (see the GHC bug report #806 at
+<https://ghc.haskell.org/trac/ghc/ticket/806>). GHC's native IO manager on
+Windows (\'WinIO\'), introduced as a preview in
+  [GHC 9.0.1](https://downloads.haskell.org/ghc/9.0.1/docs/html/users_guide/9.0.1-notes.html#highlights),
+has not yet provided a solution. On Windows, this library uses emulation based
+on the Windows Console API to try to read data emitted into the console input
+stream. Functions that use that emulation are not supported on consoles, such
+as mintty, that are not based on that API.
+
+== Function variants provided
+Every function exported by this module comes in three variants, namely:
 
  * A variant that has an @IO ()@ type and doesn't take a @Handle@ (for example,
    @clearScreen :: IO ()@). This variant just outputs the \`ANSI\` command
    directly to the standard output channel ('stdout') and any terminal
    corresponding to it. Commands issued like this should work as you expect on
-   both Unix-like operating systems and Windows.
+   both Unix-like operating systems and Windows (unless exceptions on Windows
+   are stated).
 
  * An \'@h@...\' variant that has an @IO ()@ type but takes a @Handle@ (for
    example, @hClearScreen :: Handle -> IO ()@). This variant outputs the
    \`ANSI\` command to the supplied handle and any terminal corresponding to it.
    Commands issued like this should also work as you expect on both Unix-like
-   operating systems and Windows.
+   operating systems and Windows (unless exceptions on Windows are stated).
 
  * A \'...@Code@\' variant that has a @String@ type (for example,
    @clearScreenCode :: String@). This variant outputs the sequence of control
    characters as a 'String', which can be added to any other bit of text before
-   being output. The use of these codes is generally discouraged because they
-   will not work on legacy versions of Windows where the terminal in use is not
-   ANSI-enabled (see further above). On Windows, where emulation has been
-   necessary, these variants will always output the empty string. That is done
-   so that it is possible to use them portably; for example, coloring console
-   output on the understanding that you will see colors only if you are running
-   on a Unix-like operating system or a version of Windows where emulation has
-   not been necessary. If the control characters are always required, see module
+   being output. If a high degree of backwards compatability is rewuired, the
+   use of these codes is discouraged because they will not work on legacy
+   versions of Windows where the terminal in use is not ANSI-enabled (see
+   further above). On Windows, where emulation has been necessary, these
+   variants will always output the empty string. That is done so that it is
+   possible to use them portably; for example, coloring console output on the
+   understanding that you will see colors only if you are running on a Unix-like
+   operating system or a version of Windows where emulation has not been
+   necessary. If the control characters are always required, see module
    "System.Console.ANSI.Codes".
 
-Example:
+== Examples of use
+
+A simple example is below:
 
 > module Main where
 >
@@ -102,7 +132,7 @@ Example:
 >   setSGR [Reset]  -- Reset to default colour scheme
 >   putStrLn "Default colors."
 
-Another example:
+Another example is below:
 
 > module Main where
 >
