@@ -22,9 +22,17 @@ module System.Console.ANSI.Codes
     module System.Console.ANSI.Types
 
     -- * Cursor movement by character
+    --
+    -- | These functions yield @\"\"@ when the number is @0@ as, on some
+    -- terminals, a @0@ parameter for the underlying \'ANSI\' code specifies a
+    -- default parameter of @1@.
   , cursorUpCode, cursorDownCode, cursorForwardCode, cursorBackwardCode
 
     -- * Cursor movement by line
+    --
+    -- | These functions yield the equivalent of @setCursorColumnCode 0@ when
+    -- the number is @0@ as, on some terminals, a @0@ parameter for the
+    -- underlying \'ANSI\' code specifies a default parameter of @1@.
   , cursorUpLineCode, cursorDownLineCode
 
     -- * Directly changing cursor position
@@ -39,8 +47,18 @@ module System.Console.ANSI.Codes
   , clearFromCursorToLineBeginningCode, clearLineCode
 
     -- * Scrolling the screen
+    --
+    -- | These functions yield @\"\"@ when the number is @0@ as, on some
+    -- terminals, a @0@ parameter for the underlying \'ANSI\' code specifies a
+    -- default parameter of @1@.
   , scrollPageUpCode, scrollPageDownCode
 
+    -- * Using screen buffers
+  , useAlternateScreenBufferCode, useNormalScreenBufferCode
+
+    -- * Reporting background or foreground colors
+  , reportLayerColorCode
+
     -- * Select Graphic Rendition mode: colors and other whizzy stuff
   , setSGRCode
 
@@ -56,9 +74,10 @@ module System.Console.ANSI.Codes
   , setTitleCode
 
     -- * Utilities
-  , colorToCode, csi, sgrToCode
+  , colorToCode, csi, osc, sgrToCode
   ) where
 
+import Data.Char (isPrint)
 import Data.List (intercalate)
 
 import Data.Colour.SRGB (toSRGB24, RGB (..))
@@ -75,6 +94,18 @@ csi :: [Int]  -- ^ List of parameters for the control sequence
     -> String
 csi args code = "\ESC[" ++ intercalate ";" (map show args) ++ code
 
+-- | 'osc' @parameterS parametersT@, where @parameterS@ specifies the type of
+-- operation to perform and @parametersT@ is the other parameter(s) (if any),
+-- returns the control sequence comprising the control function OPERATING SYSTEM
+-- COMMAND (OSC) followed by the parameters (separated by \';\') and ending with
+-- the STRING TERMINATOR (ST) @\"\\ESC\\\\\"@.
+--
+-- @since 0.11.4
+osc :: String -- ^ Ps parameter
+    -> String -- ^ Pt parameter(s)
+    -> String
+osc pS pT = "\ESC]" ++ pS ++ ";" ++ pT ++ "\ESC\\"
+
 -- | 'colorToCode' @color@ returns the 0-based index of the color (one of the
 -- eight colors in the ANSI standard).
 colorToCode :: Color -> Int
@@ -129,15 +160,15 @@ sgrToCode sgr = case sgr of
 cursorUpCode, cursorDownCode, cursorForwardCode, cursorBackwardCode
   :: Int -- ^ Number of lines or characters to move
   -> String
-cursorUpCode n = csi [n] "A"
-cursorDownCode n = csi [n] "B"
-cursorForwardCode n = csi [n] "C"
-cursorBackwardCode n = csi [n] "D"
+cursorUpCode n = if n == 0 then "" else csi [n] "A"
+cursorDownCode n = if n == 0 then "" else csi [n] "B"
+cursorForwardCode n = if n == 0 then "" else csi [n] "C"
+cursorBackwardCode n = if n == 0 then "" else csi [n] "D"
 
 cursorDownLineCode, cursorUpLineCode :: Int -- ^ Number of lines to move
                                      -> String
-cursorDownLineCode n = csi [n] "E"
-cursorUpLineCode n = csi [n] "F"
+cursorDownLineCode n = if n == 0 then csi [1] "G" else csi [n] "E"
+cursorUpLineCode n = if n == 0 then csi [1] "G" else csi [n] "F"
 
 -- | Code to move the cursor to the specified column. The column numbering is
 -- 0-based (that is, the left-most column is numbered 0).
@@ -164,19 +195,34 @@ restoreCursorCode = "\ESC8"
 -- Note that the information that is emitted is 1-based (the top-left corner is
 -- at row 1 column 1) but 'setCursorPositionCode' is 0-based.
 --
--- In isolation of 'getReportedCursorPosition' or 'getCursorPosition', this
--- function may be of limited use on Windows operating systems because of
--- difficulties in obtaining the data emitted into the console input stream.
--- The function 'hGetBufNonBlocking' in module "System.IO" does not work on
--- Windows. This has been attributed to the lack of non-blocking primatives in
--- the operating system (see the GHC bug report #806 at
--- <https://ghc.haskell.org/trac/ghc/ticket/806>).
+-- In isolation of 'System.Console.ANSI.getReportedCursorPosition' or
+-- 'System.Console.ANSI.getCursorPosition', this function may be of limited use
+-- on Windows operating systems because of difficulties in obtaining the data
+-- emitted into the console input stream.
 --
 -- @since 0.7.1
 reportCursorPositionCode :: String
-
 reportCursorPositionCode = csi [] "6n"
 
+-- | Code to emit the layer color into the console input stream, immediately
+-- after being recognised on the output stream, as:
+-- @ESC ] \<Ps> ; rgb: \<red> ; \<green> ; \<blue> \<ST>@
+-- where @\<Ps>@ is @10@ for 'Foreground' and @11@ for 'Background'; @\<red>@,
+-- @\<green>@ and @\<blue>@ are the color channel values in hexadecimal (4, 8,
+-- 12 and 16 bit values are possible, although 16 bit values are most common);
+-- and @\<ST>@ is the STRING TERMINATOR (ST). ST depends on the terminal
+-- software and may be the @BEL@ character or @ESC \\@ characters.
+--
+-- This function may be of limited, or no, use on Windows operating systems
+-- because (1) the control character sequence is not supported on native
+-- terminals (2) of difficulties in obtaining the data emitted into the
+-- console input stream. See 'System.Console.ANSI.getReportedLayerColor'.
+--
+-- @since 0.11.4
+reportLayerColorCode :: ConsoleLayer -> String
+reportLayerColorCode Foreground = osc "10" "?"
+reportLayerColorCode Background = osc "11" "?"
+
 clearFromCursorToScreenEndCode, clearFromCursorToScreenBeginningCode,
   clearScreenCode :: String
 clearFromCursorToLineEndCode, clearFromCursorToLineBeginningCode,
@@ -191,8 +237,12 @@ clearLineCode = csi [2] "K"
 
 scrollPageUpCode, scrollPageDownCode :: Int -- ^ Number of lines to scroll by
                                      -> String
-scrollPageUpCode n = csi [n] "S"
-scrollPageDownCode n = csi [n] "T"
+scrollPageUpCode n = if n == 0 then "" else csi [n] "S"
+scrollPageDownCode n = if n == 0 then "" else csi [n] "T"
+
+useAlternateScreenBufferCode, useNormalScreenBufferCode :: String
+useAlternateScreenBufferCode = csi [] "?1049h"
+useNormalScreenBufferCode = csi [] "?1049l"
 
 setSGRCode :: [SGR] -- ^ Commands: these will typically be applied on top of the
                     -- current console SGR mode. An empty list of commands is
@@ -218,10 +268,11 @@ hyperlinkWithParamsCode
   -> String
   -- ^ Link text
   -> String
-hyperlinkWithParamsCode ps uri link =
-  "\ESC]8;" ++ ps' ++ ";" ++ uri ++ "\ESC\\" ++ link ++ "\ESC]8;;\ESC\\"
+hyperlinkWithParamsCode params uri link =
+  osc "8" pT ++ link ++ osc "8" ";"
  where
-  ps' = intercalate ":" $ map (\(k, v) -> k ++ "=" ++ v) ps
+  pT = params' ++ ";" ++ uri
+  params' = intercalate ":" $ map (\(k, v) -> k ++ "=" ++ v) params
 
 -- | Code to introduce a hyperlink.
 --
@@ -258,4 +309,4 @@ hyperlinkWithIdCode linkId = hyperlinkWithParamsCode [("id", linkId)]
 -- behaviour between Unixes and Windows.
 setTitleCode :: String -- ^ New window title and icon name
              -> String
-setTitleCode title = "\ESC]0;" ++ filter (/= '\007') title ++ "\007"
+setTitleCode title = osc "0" (filter isPrint title)
