@@ -10,6 +10,7 @@ import Control.Exception (catchJust, IOException)
 import qualified Control.Exception as CE (catch)
 import Control.Monad (unless)
 import Data.Bits ((.&.), (.|.), complement, shiftL, shiftR)
+import Data.Char (isPrint)
 import Data.IORef (IORef, newIORef, readIORef, writeIORef)
 import Data.List (foldl', minimumBy)
 import Data.Maybe (mapMaybe)
@@ -17,14 +18,18 @@ import qualified Data.Map.Strict as Map (Map, empty, insert, lookup)
 import System.IO (Handle, hIsTerminalDevice, hPutStr, stdin)
 import System.IO.Unsafe (unsafePerformIO)
 import Text.ParserCombinators.ReadP (readP_to_S)
+import Text.Printf(printf)
 
 import Data.Colour (Colour)
 import Data.Colour.Names (black, blue, cyan, green, grey, lime, magenta, maroon,
   navy, olive, purple, red, silver, teal, white, yellow)
-import Data.Colour.SRGB (RGB (..), toSRGB)
+import Data.Colour.SRGB (toSRGB)
+#ifdef MIN_VERSION_mintty
 import System.Console.MinTTY (isMinTTYHandle)
+#else
+import System.Win32.MinTTY (isMinTTYHandle)
+#endif
 
-import System.Console.ANSI.Types
 import qualified System.Console.ANSI.Unix as Unix
 import System.Console.ANSI.Windows.Detect
 import System.Console.ANSI.Windows.Emulator.Codes
@@ -231,6 +236,9 @@ hScrollPageDown cds h n
   = emulatorFallback (Unix.hScrollPageDown h n) $ withHandle h $
       \handle -> hScrollPage cds handle n
 
+hUseAlternateScreenBuffer _ = pure ()
+hUseNormalScreenBuffer _ = pure ()
+
 {-# INLINE applyANSIColorToAttribute #-}
 applyANSIColorToAttribute :: WORD -> WORD -> WORD -> Color -> WORD -> WORD
 applyANSIColorToAttribute rED gREEN bLUE color attribute = case color of
@@ -357,7 +365,7 @@ hHyperlinkWithParams h _ _ = hPutStr h
 -- really what I'm designing for.
 hSetTitle h title
   = emulatorFallback (Unix.hSetTitle h title) $
-      withTString title $ setConsoleTitle
+      withTString (filter isPrint title) setConsoleTitle
 
 cursorPositionRef :: IORef (Map.Map HANDLE COORD)
 {-# NOINLINE cursorPositionRef #-}
@@ -374,7 +382,7 @@ hRestoreCursor h
   = emulatorFallback (Unix.hRestoreCursor h) $ withHandle h $ \handle -> do
       m <- readIORef cursorPositionRef
       let result = Map.lookup handle m
-      maybe (return ()) (setConsoleCursorPosition handle) result
+      maybe (pure ()) (setConsoleCursorPosition handle) result
 
 hReportCursorPosition h
   = emulatorFallback (Unix.hReportCursorPosition h) $ withHandle h $
@@ -387,7 +395,28 @@ hReportCursorPosition h
         hIn <- getStdHandle sTD_INPUT_HANDLE
         _ <- writeConsoleInput hIn $ keyPresses $
             "\ESC[" ++ show y ++ ";" ++ show x ++ "R"
-        return ()
+        pure ()
+
+hReportLayerColor h layer
+  = emulatorFallback (Unix.hReportLayerColor h layer) $ withHandle h $
+      \handle -> do
+        result <- getConsoleScreenBufferInfoEx handle
+        let attributes = csbix_attributes result
+            colorTable = csbix_color_table result
+            fgRef = attributes .&. fOREGROUND_INTENSE_WHITE
+            bgRef = shiftR (attributes .&. bACKGROUND_INTENSE_WHITE) 4
+            fgColor = colorTable !! fromIntegral fgRef
+            bgColor = colorTable !! fromIntegral bgRef
+            (oscCode, color) = case layer of
+              Foreground -> ("10", fgColor)
+              Background -> ("11", bgColor)
+            r = shiftL (color .&. 0xFF) 8
+            g = color .&. 0xFF00
+            b = shiftR (color .&. 0xFF0000) 8
+            report = printf "\ESC]%s;rgb:%04x/%04x/%04x\ESC\\" oscCode r g b
+        hIn <- getStdHandle sTD_INPUT_HANDLE
+        _ <- writeConsoleInput hIn $ keyPresses report
+        pure ()
 
 keyPress :: Char -> [INPUT_RECORD]
 keyPress c = [keyDown, keyUp]
@@ -442,12 +471,12 @@ hSupportsANSIWithoutEmulation handle = do
   supportsANSI <- detectHandleSupportsANSI handle  -- Without reference to the
                                                    -- environment
   case supportsANSI of
-    Just isSupported -> return (Just isSupported)
+    Just isSupported -> pure (Just isSupported)
     Nothing -> do  -- Not sure, based on the handle alone
       notDumb <- isNotDumb  -- Test the environment
       if notDumb
-        then return Nothing  -- Still not sure!
-        else return (Just False) -- A dumb terminal
+        then pure Nothing  -- Still not sure!
+        else pure (Just False) -- A dumb terminal
 
 -- Borrowed from an HSpec patch by Simon Hengel
 -- (https://github.com/hspec/hspec/commit/d932f03317e0e2bd08c85b23903fb8616ae642bd)
@@ -462,18 +491,20 @@ isNotDumb = (/= Just "dumb") . lookup "TERM" <$> getEnvironment
 
 -- getReportedCursorPosition :: IO String
 -- (See Common-Include.hs for Haddock documentation)
-getReportedCursorPosition
-  = CE.catch getReportedCursorPosition' getCPExceptionHandler
+getReportedCursorPosition = getReported
+
+getReported :: IO String
+getReported = CE.catch getReported' getReportedExceptionHandler
  where
-  getReportedCursorPosition' = withHandleToHANDLE stdin action
+  getReported' = withHandleToHANDLE stdin action
    where
     action hdl = do
       n <- getNumberOfConsoleInputEvents hdl
       if n == 0
-        then return ""
+        then pure ""
         else do
           es <- readConsoleInput hdl n
-          return $ stringFromInputEvents es
+          pure $ stringFromInputEvents es
     stringFromInputEvents = cWcharsToChars . wCharsFromInputEvents
     wCharsFromInputEvents = mapMaybe wCharFromInputEvent
     wCharFromInputEvent e = if isKeyEvent && isKeyDown
@@ -493,25 +524,40 @@ getReportedCursorPosition
 hGetCursorPosition h = fmap to0base <$> getCursorPosition'
  where
   to0base (row, col) = (row - 1, col - 1)
-  getCursorPosition' = CE.catch getCursorPosition'' getCPExceptionHandler
+  getCursorPosition' =
+    hGetReport h hReportCursorPosition getReportedCursorPosition cursorPosition
+
+hGetReport :: Handle
+           -> (Handle -> IO ())
+           -> IO String -> ReadP a -> IO (Maybe a)
+hGetReport h report get parse =
+  CE.catch getReport getReportedExceptionHandler
    where
-    getCursorPosition'' = do
+    getReport = do
       withHandleToHANDLE stdin flush -- Flush the console input buffer
-      hReportCursorPosition h
+      report h
       hFlush h -- ensure the report cursor position code is sent to the
                -- operating system
-      input <- getReportedCursorPosition
-      case readP_to_S cursorPosition input of
-        [] -> return Nothing
-        [((row, col),_)] -> return $ Just (row, col)
-        (_:_) -> return Nothing
+      input <- get
+      case readP_to_S parse input of
+        [] -> pure Nothing
+        [(value,_)] -> pure $ Just value
+        (_:_) -> pure Nothing
      where
       flush hdl = do
         n <- getNumberOfConsoleInputEvents hdl
         unless (n == 0) (void $ readConsoleInput hdl n)
 
-getCPExceptionHandler :: IOException -> IO a
-getCPExceptionHandler e = error msg
+
+-- getReportedLayerColor :: ConsoleLayer -> IO String
+--(See Common-Include.hs for Haddock documentation)
+getReportedLayerColor _ = getReported
+
+hGetLayerColor h layer = hGetReport h
+  (`hReportLayerColor` layer) (getReportedLayerColor layer) (layerColor layer)
+
+getReportedExceptionHandler :: IOException -> IO a
+getReportedExceptionHandler e = error msg
  where
   msg = "Error: " ++ show e ++ "\nThis error may be avoided by using a " ++
         "console based on the Windows' Console API, such as Command Prompt " ++
