@@ -18,9 +18,10 @@ module System.Console.ANSI.Windows.Foreign
 
     charToWCHAR, cWcharsToChars,
 
-    COORD(..), SMALL_RECT(..), rect_top, rect_bottom, rect_left, rect_right,
-    rect_width, rect_height, CONSOLE_CURSOR_INFO(..),
-    CONSOLE_SCREEN_BUFFER_INFO(..), CHAR_INFO(..),
+    COLORREF, COORD(..), SMALL_RECT(..), rect_top, rect_bottom, rect_left,
+    rect_right, rect_width, rect_height, CONSOLE_CURSOR_INFO(..),
+    CONSOLE_SCREEN_BUFFER_INFO(..), CONSOLE_SCREEN_BUFFER_INFOEX(..),
+    CHAR_INFO(..),
 
     sTD_INPUT_HANDLE, sTD_OUTPUT_HANDLE, sTD_ERROR_HANDLE,
 
@@ -34,6 +35,7 @@ module System.Console.ANSI.Windows.Foreign
 
     getStdHandle,
     getConsoleScreenBufferInfo,
+    getConsoleScreenBufferInfoEx,
     getConsoleCursorInfo,
     getConsoleMode,
 
@@ -52,22 +54,20 @@ module System.Console.ANSI.Windows.Foreign
     ConsoleException (..)
   ) where
 
-#if !MIN_VERSION_base(4,8,0)
-import Control.Applicative ((<$>), (<*>))
-#endif
 import Control.Exception (Exception, throw)
 import Data.Bits ((.|.), shiftL)
 import Data.Char (chr, ord)
 import Data.Typeable (Typeable)
+import Data.Word (Word32)
 import Foreign.C.Types (CInt (..), CWchar (..))
 import Foreign.Marshal.Alloc (alloca)
-import Foreign.Marshal.Array (allocaArray, peekArray, withArrayLen)
+import Foreign.Marshal.Array (allocaArray, peekArray, pokeArray, withArrayLen)
 import Foreign.Marshal.Utils (maybeWith, with)
 import Foreign.Ptr (Ptr, castPtr, plusPtr)
 import Foreign.Storable (Storable (..))
 -- `SHORT` and `withHandleToHANDLE` are not both available before Win32-2.5.1.0
 import System.Win32.Compat (BOOL, DWORD, ErrCode, HANDLE, LPCTSTR, LPDWORD,
-  SHORT, TCHAR, UINT, WORD, failIfFalse_, getLastError, iNVALID_HANDLE_VALUE,
+  SHORT, TCHAR, UINT, ULONG, WORD, failIfFalse_, getLastError, iNVALID_HANDLE_VALUE,
   nullHANDLE, withHandleToHANDLE, withTString)
 
 #if defined(i386_HOST_ARCH)
@@ -100,12 +100,12 @@ unpackCOORD (COORD x y)
 peekAndOffset :: Storable a => Ptr a -> IO (a, Ptr b)
 peekAndOffset ptr = do
   item <- peek ptr
-  return (item, ptr `plusPtr` sizeOf item)
+  pure (item, ptr `plusPtr` sizeOf item)
 
 pokeAndOffset :: Storable a => Ptr a -> a -> IO (Ptr b)
 pokeAndOffset ptr item = do
   poke ptr item
-  return (ptr `plusPtr` sizeOf item)
+  pure (ptr `plusPtr` sizeOf item)
 
 data COORD = COORD
   { coord_x :: SHORT
@@ -122,7 +122,7 @@ instance Storable COORD where
     let ptr' = castPtr ptr :: Ptr SHORT
     x <- peekElemOff ptr' 0
     y <- peekElemOff ptr' 1
-    return (COORD x y)
+    pure (COORD x y)
   poke ptr (COORD x y) = do
     let ptr' = castPtr ptr :: Ptr SHORT
     pokeElemOff ptr' 0 x
@@ -153,7 +153,7 @@ instance Storable SMALL_RECT where
     let ptr' = castPtr ptr :: Ptr COORD
     tl <- peekElemOff ptr' 0
     br <- peekElemOff ptr' 1
-    return (SMALL_RECT tl br)
+    pure (SMALL_RECT tl br)
   poke ptr (SMALL_RECT tl br) = do
     let ptr' = castPtr ptr :: Ptr COORD
     pokeElemOff ptr' 0 tl
@@ -170,7 +170,7 @@ instance Storable CONSOLE_CURSOR_INFO where
   peek ptr = do
     (size, ptr') <- peekAndOffset (castPtr ptr)
     visible <- peek ptr'
-    return (CONSOLE_CURSOR_INFO size visible)
+    pure (CONSOLE_CURSOR_INFO size visible)
   poke ptr (CONSOLE_CURSOR_INFO size visible) = do
     ptr' <- pokeAndOffset (castPtr ptr) size
     poke ptr' visible
@@ -195,7 +195,7 @@ instance Storable CONSOLE_SCREEN_BUFFER_INFO where
     (attributes, ptr3) <- peekAndOffset ptr2
     (window, ptr4) <- peekAndOffset ptr3
     maximum_window_size <- peek ptr4
-    return (CONSOLE_SCREEN_BUFFER_INFO
+    pure (CONSOLE_SCREEN_BUFFER_INFO
       size cursor_position attributes window maximum_window_size)
   poke ptr (CONSOLE_SCREEN_BUFFER_INFO
     size cursor_position attributes window maximum_window_size)
@@ -206,6 +206,68 @@ instance Storable CONSOLE_SCREEN_BUFFER_INFO where
       ptr4 <- pokeAndOffset ptr3 window
       poke ptr4 maximum_window_size
 
+data CONSOLE_SCREEN_BUFFER_INFOEX = CONSOLE_SCREEN_BUFFER_INFOEX
+  { csbix_size                 :: COORD
+  , csbix_cursor_position      :: COORD
+  , csbix_attributes           :: WORD
+  , csbix_window               :: SMALL_RECT
+  , csbix_maximum_window_size  :: COORD
+  , csbix_popup_attributes     :: WORD
+  , csbix_fullscreen_supported :: BOOL
+  , csbix_color_table          :: [COLORREF]
+  } deriving (Show)
+
+-- When specifying an explicit RGB color, the COLORREF value has the following
+-- hexadecimal form:
+-- 0x00bbggrr
+-- The low-order byte contains a value for the relative intensity of red; the
+-- second byte contains a value for green; and the third byte contains a value
+-- for blue. The high-order byte must be zero. The maximum value for a single
+-- byte is 0xFF.
+type COLORREF = Word32
+
+instance Storable CONSOLE_SCREEN_BUFFER_INFOEX where
+  sizeOf ~(CONSOLE_SCREEN_BUFFER_INFOEX
+    size cursor_position attributes window maximum_window_size popup_attributes
+      fullscreen_supported _)
+    = sizeOf sizeCsbix + sizeOf size + sizeOf cursor_position + sizeOf attributes + sizeOf window
+        + sizeOf maximum_window_size + sizeOf popup_attributes
+        + sizeOf fullscreen_supported + 16 * sizeOf (undefined :: COLORREF)
+  alignment ~(CONSOLE_SCREEN_BUFFER_INFOEX _ _ _ _ _ _ _ _) = alignment sizeCsbix
+  peek ptr = do
+    let ptr0 = castPtr ptr `plusPtr` sizeOf sizeCsbix
+    (size, ptr1) <- peekAndOffset ptr0
+    (cursor_position, ptr2) <- peekAndOffset ptr1
+    (attributes, ptr3) <- peekAndOffset ptr2
+    (window, ptr4) <- peekAndOffset ptr3
+    (maximum_window_size, ptr5) <- peekAndOffset ptr4
+    (popup_attributes, ptr6) <- peekAndOffset ptr5
+    (fullscreen_supported, ptr7) <- peekAndOffset ptr6
+    color_table <- peekArray 16 ptr7
+    pure (CONSOLE_SCREEN_BUFFER_INFOEX
+      size cursor_position attributes window maximum_window_size
+      popup_attributes fullscreen_supported color_table)
+  poke ptr (CONSOLE_SCREEN_BUFFER_INFOEX
+    size cursor_position attributes window maximum_window_size popup_attributes
+    fullscreen_supported color_table)
+    = do
+      ptr0 <- pokeAndOffset (castPtr ptr) sizeCsbix
+      ptr1 <- pokeAndOffset ptr0 size
+      ptr2 <- pokeAndOffset ptr1 cursor_position
+      ptr3 <- pokeAndOffset ptr2 attributes
+      ptr4 <- pokeAndOffset ptr3 window
+      ptr5 <- pokeAndOffset ptr4 maximum_window_size
+      ptr6 <- pokeAndOffset ptr5 popup_attributes
+      ptr7 <- pokeAndOffset ptr6 fullscreen_supported
+      pokeArray ptr7 color_table'
+   where
+    color_table' = take 16 $ color_table ++ repeat 0
+
+sizeCsbix :: ULONG
+sizeCsbix = fromIntegral $
+  sizeOf (undefined :: CONSOLE_SCREEN_BUFFER_INFOEX)
+
+
 data CHAR_INFO = CHAR_INFO
   { ci_char       :: WCHAR
   , ci_attributes :: WORD
@@ -217,7 +279,7 @@ instance Storable CHAR_INFO where
   peek ptr = do
     (char, ptr') <- peekAndOffset (castPtr ptr)
     attributes <- peek ptr'
-    return (CHAR_INFO char attributes)
+    pure (CHAR_INFO char attributes)
   poke ptr (CHAR_INFO char attributes) = do
     ptr' <- pokeAndOffset (castPtr ptr) char
     poke ptr' attributes
@@ -265,6 +327,10 @@ foreign import WINDOWS_CCONV unsafe "windows.h GetConsoleScreenBufferInfo"
   cGetConsoleScreenBufferInfo :: HANDLE
                               -> Ptr CONSOLE_SCREEN_BUFFER_INFO
                               -> IO BOOL
+foreign import WINDOWS_CCONV unsafe "windows.h GetConsoleScreenBufferInfoEx"
+  cGetConsoleScreenBufferInfoEx :: HANDLE
+                                -> Ptr CONSOLE_SCREEN_BUFFER_INFOEX
+                                -> IO BOOL
 foreign import WINDOWS_CCONV unsafe "windows.h GetConsoleCursorInfo"
   cGetConsoleCursorInfo :: HANDLE -> Ptr CONSOLE_CURSOR_INFO -> IO BOOL
 foreign import WINDOWS_CCONV unsafe "windows.h GetConsoleMode"
@@ -338,7 +404,7 @@ throwIfFalse action = do
   if not succeeded
     then getLastError >>= throw . ConsoleException -- TODO: Check if last error
     -- is zero for some instructable reason (?)
-    else return ()
+    else pure ()
 
 getConsoleScreenBufferInfo :: HANDLE -> IO CONSOLE_SCREEN_BUFFER_INFO
 getConsoleScreenBufferInfo handle
@@ -347,6 +413,17 @@ getConsoleScreenBufferInfo handle
         cGetConsoleScreenBufferInfo handle ptr_console_screen_buffer_info
       peek ptr_console_screen_buffer_info
 
+getConsoleScreenBufferInfoEx :: HANDLE -> IO CONSOLE_SCREEN_BUFFER_INFOEX
+getConsoleScreenBufferInfoEx handle
+  = alloca $ \ptr_console_screen_buffer_infoex -> do
+      -- In the Windows Console API, the `CONSOLE_SCREEN_BUFFER_INFOEX`
+      -- structure passed to the `GetConsoleScreenBufferInfoEx` function must
+      -- include the size of the structure.
+      poke (castPtr ptr_console_screen_buffer_infoex) sizeCsbix
+      throwIfFalse $
+        cGetConsoleScreenBufferInfoEx handle ptr_console_screen_buffer_infoex
+      peek ptr_console_screen_buffer_infoex
+
 getConsoleCursorInfo :: HANDLE -> IO CONSOLE_CURSOR_INFO
 getConsoleCursorInfo handle = alloca $ \ptr_console_cursor_info -> do
   throwIfFalse $ cGetConsoleCursorInfo handle ptr_console_cursor_info
@@ -603,7 +680,7 @@ instance Storable INPUT_RECORD where
           -> InputFocusEvent            <$> (`peekByteOff` 4) ptr
       _ -> error $ "peek (INPUT_RECORD): Unknown event type " ++
              show evType
-    return $ INPUT_RECORD evType event
+    pure $ INPUT_RECORD evType event
   poke ptr val = do
     (`pokeByteOff` 0) ptr $ inputEventType val
     case inputEvent val of
