@@ -10,10 +10,12 @@
   , cursorDown
   , cursorForward
   , cursorBackward
+    -- ** \'h...\' variants
   , hCursorUp
   , hCursorDown
   , hCursorForward
   , hCursorBackward
+    -- ** \'...Code\' variants
   , cursorUpCode
   , cursorDownCode
   , cursorForwardCode
@@ -24,57 +26,65 @@
     -- that @*Line@ functions additionally move the cursor to the start of the
     -- line, while functions like @cursorUp@ and @cursorDown@ keep the column
     -- the same.
-    --
-    -- Also keep in mind that @*Line@ functions are not as portable. See
-    -- <https://github.com/UnkindPartition/ansi-terminal/issues/10> for the details.
   , cursorUpLine
   , cursorDownLine
+    -- ** \'h...\' variants
   , hCursorUpLine
   , hCursorDownLine
+    -- ** \'...Code\' variants
   , cursorUpLineCode
   , cursorDownLineCode
 
     -- * Directly changing cursor position
   , setCursorColumn
-  , hSetCursorColumn
-  , setCursorColumnCode
-
   , setCursorPosition
+    -- ** \'h...\' variants
+  , hSetCursorColumn
   , hSetCursorPosition
+    -- ** \'...Code\' variants
+  , setCursorColumnCode
   , setCursorPositionCode
 
     -- * Saving, restoring and reporting cursor position
+    -- | These code sequences are not part of the ECMA-48 standard; they are
+    -- popular, but non-portable extensions. On Unix-like operating systems,
+    -- they correspond to @rc@ and @sc@ capabilities in the @terminfo@ database.
+    --
+    -- Cursor positions are relative to the viewport, not to its content.
+    --
   , saveCursor
-  , hSaveCursor
-  , saveCursorCode
-
   , restoreCursor
-  , hRestoreCursor
-  , restoreCursorCode
-
   , reportCursorPosition
+    -- ** \'h...\' variants
+  , hSaveCursor
+  , hRestoreCursor
   , hReportCursorPosition
+    -- ** \'...Code\' variants
+  , saveCursorCode
+  , restoreCursorCode
   , reportCursorPositionCode
 
     -- * Clearing parts of the screen
-    -- | Note that these functions only clear parts of the screen. They do not move the
-    -- cursor.
+    -- | Note that these functions only clear parts of the screen. They do not
+    -- move the cursor. Some functions are based on the whole screen and others
+    -- are based on the line in which the cursor is located.
   , clearFromCursorToScreenEnd
   , clearFromCursorToScreenBeginning
   , clearScreen
-  , hClearFromCursorToScreenEnd
-  , hClearFromCursorToScreenBeginning
-  , hClearScreen
-  , clearFromCursorToScreenEndCode
-  , clearFromCursorToScreenBeginningCode
-  , clearScreenCode
-
   , clearFromCursorToLineEnd
   , clearFromCursorToLineBeginning
   , clearLine
+    -- ** \'h...\' variants
+  , hClearFromCursorToScreenEnd
+  , hClearFromCursorToScreenBeginning
+  , hClearScreen
   , hClearFromCursorToLineEnd
   , hClearFromCursorToLineBeginning
   , hClearLine
+    -- ** \'...Code\' variants
+  , clearFromCursorToScreenEndCode
+  , clearFromCursorToScreenBeginningCode
+  , clearScreenCode
   , clearFromCursorToLineEndCode
   , clearFromCursorToLineBeginningCode
   , clearLineCode
@@ -82,36 +92,67 @@
     -- * Scrolling the screen
   , scrollPageUp
   , scrollPageDown
+    -- ** \'h...\' variants
   , hScrollPageUp
   , hScrollPageDown
+    -- ** \'...Code\' variants
   , scrollPageUpCode
   , scrollPageDownCode
 
+    -- * Using screen buffers
+    -- | These code sequences are not part of the ECMA-48 standard; they are
+    -- popular, but non-portable extensions. On Unix-like operating systems,
+    -- they correspond to the @smcup@ and @rmcup@ capabilities in the @terminfo@
+    -- database. Windows Terminal supports them. On Windows, if emulation is
+    -- required, switching between alternate and normal screen buffers is not
+    -- emulated.
+  , useAlternateScreenBuffer
+  , useNormalScreenBuffer
+    -- ** \'h...\' variants
+  , hUseAlternateScreenBuffer
+  , hUseNormalScreenBuffer
+    -- ** \'...Code\' variants
+  , useAlternateScreenBufferCode
+  , useNormalScreenBufferCode
+
+    -- * Reporting the background or foreground colors
+  , reportLayerColor
+  , hReportLayerColor
+  , reportLayerColorCode
+
     -- * Select Graphic Rendition mode: colors and other whizzy stuff
   , setSGR
   , hSetSGR
   , setSGRCode
 
     -- * Cursor visibilty changes
+    -- | These code sequences are not part of the ECMA-48 standard; they are
+    -- popular, but non-portable extensions. In practice, many terminals support
+    -- them.
   , hideCursor
   , showCursor
+    -- ** \'h...\' variants
   , hHideCursor
   , hShowCursor
+    -- ** \'...Code\' variants
   , hideCursorCode
   , showCursorCode
 
     -- * Hyperlinks
-    -- | Some, but not all, terminals support hyperlinks - that is, clickable
-    -- text that points to a URI. On Windows, if emulation is required,
-    -- hyperlinks are not emulated.
+    -- | These code sequences are not part of the ECMA-48 standard or an Xterm
+    -- extension. However, in practice, many terminals support them, including
+    -- Windows Terminal. On Windows, if emulation is required, hyperlinks are
+    -- not emulated.
   , hyperlink
-  , hHyperlink
-  , hyperlinkCode
   , hyperlinkWithId
-  , hHyperlinkWithId
-  , hyperlinkWithIdCode
   , hyperlinkWithParams
+    -- ** \'h...\' variants
+  , hHyperlink
+  , hHyperlinkWithId
   , hHyperlinkWithParams
+    -- ** \'...Code\' variants
+  , hyperlinkCode
+  , hyperlinkWithIdCode
   , hyperlinkWithParamsCode
 
     -- * Changing the title
@@ -133,3 +174,9 @@
     -- * Getting the terminal size
   , getTerminalSize
   , hGetTerminalSize
+
+    -- * Getting the background or foreground colors
+  , getLayerColor
+  , hGetLayerColor
+  , getReportedLayerColor
+  , layerColor
